/*
Martrikelnummer der Gruppenmitglieder: Olha Grubryn (235122), Egor Rutkowski(225064), Nikola Kramaric (238498), Dennis Ruppel (238503)
*/
package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.*;

import java.util.ArrayList;


/**
 * AkkaMainSystem Actor class that initializes the system with customers, an address book, and a distribution center.
 */
public class AkkaMainSystem extends AbstractBehavior<AkkaMainSystem.Create> {

    /**
     * Message to trigger the creation of the AkkaMainSystem.
     */
    public static class Create {}

    /**
     * Factory method to create a new AkkaMainSystem actor.
     *
     * @return the behavior of the AkkaMainSystem actor
     */
    public static Behavior<Create> create() {
        return Behaviors.setup(AkkaMainSystem::new);
    }

    // Static reference to the DistributionCenter actor
    private static ActorRef<DistributionCenter.Message> distributionCenter;

    /**
     * Static method to retrieve the reference of the DistributionCenter actor.
     *
     * @return the DistributionCenter actor reference
     */
    public static ActorRef<DistributionCenter.Message> getDistributionCenter() {
        return distributionCenter;
    }

    /**
     * Constructor for AkkaMainSystem.
     * Initializes the system by creating customers, an address book, and a distribution center.
     *
     * @param context the actor context
     */
    private AkkaMainSystem(ActorContext<Create> context) {
        super(context);

        // Creating the list of customer actors
        ArrayList<ActorRef<Customer.Message>> customers = new ArrayList<>();
        customers.add(getContext().spawn(Customer.create("Alice"), "alice"));
        customers.add(getContext().spawn(Customer.create("Bob"), "bob"));
        customers.add(getContext().spawn(Customer.create("Charles"), "charles"));
        customers.add(getContext().spawn(Customer.create("Derick"), "derick"));

        // Creating the AddressBook actor with the list of customers
        ActorRef<AddressBook.Message> addressBook = getContext().spawn(AddressBook.create(customers), "addressBook");

        // Creating the DistributionCenter actor with the address book reference
        distributionCenter = getContext().spawn(DistributionCenter.create(addressBook), "distributionCenter");
    }

    @Override
    public Receive<Create> createReceive() {
        return newReceiveBuilder()
                .onMessage(Create.class, this::onCreate)
                .build();
    }

    /**
     * Handles the Create message. Additional setup can be done here if needed.
     *
     * @param command the create command
     * @return the behavior of the AkkaMainSystem actor
     */
    private Behavior<Create> onCreate(Create command) {
        // Additional setup if needed
        return this;
    }
}
