package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.*;
import akka.io.dns.internal.Message;

import java.util.ArrayList;
import java.util.Random;


import static com.example.AkkaMainSystem.*;

public class ExampleActor extends AbstractBehavior<ExampleActor.Message> {

    public interface Message {}

    public record ExampleMessage(ActorRef<Create> someReference, String someString) implements Message {  }
    public record Customer(String name, ActorRef<akka.io.dns.internal.Message> reference) {}

    // Nachricht, um eine Empfängerreferenz hinzuzufügen
    public static class AddCustomer implements Message {
        private final ActorRef<Message> customer;

        public AddCustomer(ActorRef<Message> customer) {
            this.customer = customer;
        }

        public ActorRef<Message> getCustomer() {
            return customer;
        }
    }

    private final String name;

    public static class PrintCustomers implements Message {}
    private final ArrayList<ActorRef<Message>> customers; // ArrayList aller Empfänger*innen
    private final Random random;
    public static class PrintRandomCustomer implements Message {}// Nachricht, um einen zufälligen Kunden auszugeben



    private ExampleActor(ActorContext<Message> context, String name) {
        super(context);
        this.name = name;
        this.customers = new ArrayList<>();
        this.random = new Random();
    }

    public static Behavior<Message> create(String name) {
        return Behaviors.setup(context -> new ExampleActor(context, name));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(ExampleMessage.class, this::onExampleMessage)
                .onMessage(AddCustomer.class, this::onAddCustomer)
                .onMessage(PrintCustomers.class, this::onPrintCustomers)
                .onMessage(PrintRandomCustomer.class, this::onPrintRandomCustomer)
                .build();
    }

    private Behavior<Message> onExampleMessage(ExampleMessage msg) {
        getContext().getLog().info("I ({}) got a message: ExampleMessage({},{})", this.name, msg.someReference, msg.someString);
        return this;
    }


    private Behavior<Message> onAddCustomer(AddCustomer msg) {
        customers.add(msg.getCustomer());
        return this;
    }

    private Behavior<Message> onPrintCustomers(PrintCustomers msg) {
        for (ActorRef<Message> customer : customers) {
            getContext().getLog().info("Customer: {}", customer.path().name());
        }
        return this;
    }
    private Behavior<Message> onPrintRandomCustomer(PrintRandomCustomer msg) {
        if (!customers.isEmpty()) {
            int randomIndex = random.nextInt(customers.size());
            ActorRef<Message> randomCustomer = customers.get(randomIndex);
            getContext().getLog().info("Random Customer: {}", randomCustomer.path().name());
        } else {
            getContext().getLog().info("No customers available.");
        }
        return this;
    }
}